#define __GM_DATA_C__
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <earthworm.h>
#include "config.h"
#include "gm_data.h"

static int largest_seconds = 0;
static int data_table_size = 0;

int decrement_data_index(int *index)
{
   if (--*index < 0)
   {
      *index = data_table_size - 1;
   }
   return *index;
}

int increment_data_index(int *index)
{
   if (++*index >= data_table_size)
   {
      *index = 0;
   }
   return *index;
}

void get_largest_pg_values(PG_TABLE_ENTRY *largest_pg_values, int time_span, STATION *Sta)
{
   double pga, pgv;
   int i;
   int index = Sta->last_saved_index;
   int numIntervals = time_span / Gparm.OutputInterval;
   largest_pg_values->pga = Sta->pg_table[index].pga;
   largest_pg_values->pgv = Sta->pg_table[index].pgv;
   for (i = 1; i < numIntervals; i++)
   {
      decrement_data_index(&index);
      pga = Sta->pg_table[index].pga;
      pgv = Sta->pg_table[index].pgv;
      if (pga > largest_pg_values->pga)
      {
         largest_pg_values->pga = pga;
      }
      if (pgv > largest_pg_values->pgv)
      {
         largest_pg_values->pgv = pgv;
      }
   }
}

int init_gm_data()
{
   if (data_table_size == 0)
   {
      // determine the larget window
      int index;
      int seconds;
      for (index = 0; index < Gparm.NumOutputWindows; index++)
      {
         seconds = Gparm.OutputWindows[index];
         if (seconds > largest_seconds)
         {
            largest_seconds = seconds;
         }
      }
      data_table_size = largest_seconds / Gparm.OutputInterval;
      if (Gparm.Debug)
      {
         printf("report_gm: Gparm.OutputInterval=%d\n", Gparm.OutputInterval);
         printf("report_gm: largest_seconds=%d\n", largest_seconds);
         printf("report_gm: data_table_size=%d\n", data_table_size);
      }
   }
   return data_table_size;
}

void save_values(time_t timeNow, STATION *Sta, double time, double pga, double pgv)
{
   int current_index = (timeNow / Gparm.OutputInterval) % data_table_size;

   // if same window as last time
   if (current_index == Sta->last_saved_index)
   {
      // udpdate value if larger
      if (pga > Sta->pg_table[current_index].pga)
      {
         Sta->pg_table[current_index].pga = pga;
         if ( time > Sta->time)
         {
            Sta->time = time;
            Sta->latency = timeNow - time;
         }
         if (Gparm.Debug > 1)
         {
            printf("report_gm: save pga value(%s.%s.%s.%s, %f, %d, %f)\n", 
                   Sta->sta, Sta->chan, Sta->net, Sta->loc, time, current_index, pga);
         }
      }
      if (pgv > Sta->pg_table[current_index].pgv)
      {
         Sta->pg_table[current_index].pgv = pgv;
         if ( time > Sta->time)
         {
            Sta->time = time;
            Sta->latency = timeNow - time;
         }
         if (Gparm.Debug > 1)
         {
            printf("report_gm: save pgv value(%s.%s.%s.%s, %f, %d, %f)\n", 
                   Sta->sta, Sta->chan, Sta->net, Sta->loc, time, current_index, pgv);
         }
      }
   }
   else
   {
      // if not first value saved
      if (Sta->last_saved_index != -1)
      {
         // clear previous table values if gap
         int index = Sta->last_saved_index;;
         while ((index = increment_data_index(&index)) != current_index)
         {
            if (Gparm.Debug > 1)
            {
               printf("report_gm: clearing(%s.%s.%s.%s, %d)\n", 
                      Sta->sta, Sta->chan, Sta->net, Sta->loc, index);
            }
            memset(Sta->pg_table + current_index, 0, sizeof(PG_TABLE_ENTRY));
         }
      }
      if (Gparm.Debug > 1)
      {
         printf("report_gm: save_values(%s.%s.%s.%s, %f, %d, %f, %f)\n", 
                Sta->sta, Sta->chan, Sta->net, Sta->loc, time, current_index, pga, pgv);
      }
      Sta->pg_table[current_index].pga = pga;
      Sta->pg_table[current_index].pgv = pgv;
      if ( time > Sta->time)
      {
         Sta->time = time;
         Sta->latency = timeNow - time;
      }
      Sta->last_saved_index = current_index;
   }
}
